using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Threading;
using System.Threading.Tasks;
using Autofac;
using FitConnect;
using FitConnect.Models;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using MockContainer;
using Newtonsoft.Json;
using NUnit.Framework;

namespace IntegrationTests.Sender;

/// <summary>
///     This test runs multi sender and one subscriber which "cleans" the server
///     so other subscriber won't get any submissions.
/// </summary>
[Ignore("Can not be combined with other tests - need to be run separately")]
[TestFixture]
public class ThreadTest {
    [SetUp]
    public void Setup() {
        _container = Container.Create();
        _setting = _container.Resolve<IFitConnectSettings>();
        _logger = LoggerFactory.Create(b => b.AddConsole())
            .CreateLogger<FitConnect.Sender>();
    }

    private IContainer _container = null!;
    private IFitConnectSettings _setting = null!;
    private ILogger _logger = null!;
    private const int NumberOfThreads = 34;

    [Test]
    [Order(1)]
    public void ThreadedSender() {
        var tasks = new List<Task>();

        for (var i = 0; i < NumberOfThreads; i++)
            tasks.Add(Task.Run(() => {
                var counter = Thread.CurrentThread.ManagedThreadId;
                var sender = new FitConnect.Sender(
                    FitConnectEnvironment.Testing,
                    _setting.SenderClientId, _setting.SenderClientSecret, _logger);

                var delayed = SubmissionBuilder.WithDestination(_setting.DestinationId)
                    .WithServiceType($"ThreadTest_{counter}", _setting.LeikaKey);

                Thread.Sleep(RandomNumberGenerator.GetInt32(100, 500));
                var outgoing = delayed
                    .WithJsonData(JsonConvert.SerializeObject(new {
                        Name = $"ThreadTest_{counter}",
                        ThreadId = counter
                    }))
                    .WithAttachments(new Attachment("Test.pdf", $"Attachment_{counter}"))
                    .Build();

                sender.SendAsync(outgoing).Wait();
            }));

        while (!tasks.All(t => t.IsCompleted)) {
        }
    }

    [Test]
    [Order(2)]
    public void GetSubmissions() {
        var subscriber = Client.GetSubscriber(FitConnectEnvironment.Testing,
            _setting.SubscriberClientId, _setting.SubscriberClientSecret,
            _setting.PrivateKeyDecryption, _setting.PrivateKeySigning, _logger);

        var submissions = subscriber.GetAvailableSubmissionsAsync().Result.ToList();

        foreach (var submission in submissions)
            subscriber
                .RequestSubmissionAsync(submission.Id!).Result
                .AcceptSubmissionAsync().Wait();

        submissions.Count.Should().Be(NumberOfThreads);
    }
}
